import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='tfm',
    version='0.3.3-ALPHA',
    packages=['tfm', ],
    license='GPLv3',
    author='Till Mahlburg',
    # author_email="author@example.com",
    description='A simple and Qt file manager for Linux',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/tmahlburg/tfm',
    install_requires=[
        'pyside6',
        'prefixed',
        'send2trash',
        'pyudev',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: X11 Applications :: Qt',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3'
        + ' (GPLv3)',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Desktop Environment :: File Managers',
    ],
    python_requires='>=3.7',
)
